@@ -1,3 +1,4 @@
+{-# LANGUAGE BangPatterns               #-}
 {-# LANGUAGE DeriveFunctor              #-}
 {-# LANGUAGE EmptyDataDecls             #-}
 {-# LANGUAGE FlexibleContexts           #-}
@@ -6,6 +7,7 @@
 {-# LANGUAGE GeneralizedNewtypeDeriving #-}
 {-# LANGUAGE MultiParamTypeClasses      #-}
 {-# LANGUAGE StandaloneDeriving         #-}
+{-# LANGUAGE TemplateHaskell            #-}
 {-# LANGUAGE TypeFamilies               #-}
 {-# LANGUAGE TypeOperators              #-}
 {-# LANGUAGE UndecidableInstances       #-}
@@ -54,28 +56,28 @@ module Diagrams.Segment
          -- $segmeas
 
        , SegCount(..)
-       , ArcLength(..), getArcLengthCached, getArcLengthFun, getArcLengthBounded
+       , ArcLength(..)
+       , getArcLengthCached, getArcLengthFun, getArcLengthBounded
        , TotalOffset(..)
-       , OffsetEnvelope(..)
+       , OffsetEnvelope(..), oeOffset, oeEnvelope
        , SegMeasure
 
        ) where
 
+import           Control.Lens (makeLenses, Wrapped(..), Rewrapped, iso, op)
 import           Control.Applicative (liftA2)
 import           Data.AffineSpace
-import           Data.Default.Class
 import           Data.FingerTree
 import           Data.Monoid.MList
 import           Data.Semigroup
 import           Data.VectorSpace    hiding (Sum (..))
-import           Numeric.Interval    (Interval (..))
-import qualified Numeric.Interval    as I
+import           Numeric.Interval.Kaucher    (Interval (..))
+import qualified Numeric.Interval.Kaucher    as I
 
 import           Diagrams.Core
 import           Diagrams.Located
 import           Diagrams.Parametric
 import           Diagrams.Solve
-import           Diagrams.Util
 
 
 ------------------------------------------------------------
@@ -102,7 +104,7 @@ data Closed
 --   a fixed offset from its start.
 data Offset c v where
   OffsetOpen   :: Offset Open v
-  OffsetClosed :: v -> Offset Closed v
+  OffsetClosed :: !v -> Offset Closed v
 
 deriving instance Show v => Show (Offset c v)
 deriving instance Eq   v => Eq   (Offset c v)
@@ -129,10 +131,10 @@ instance HasLinearMap v => Transformable (Offset c v) where
 --   however, affected by other transformations such as rotations and
 --   scales.
 data Segment c v
-    = Linear (Offset c v)
+    = Linear !(Offset c v)
       -- ^ A linear segment with given offset.
 
-    | Cubic v v (Offset c v)
+    | Cubic !v !v !(Offset c v)
       -- ^ A cubic Bézier segment specified by
       --   three offsets from the starting
       --   point to the first control point,
@@ -231,7 +233,7 @@ segOffset (Cubic _ _ (OffsetClosed v)) = v
 instance (InnerSpace v, OrderedField (Scalar v)) => Enveloped (Segment Closed v) where
 
   getEnvelope (s@(Linear {})) = mkEnvelope $ \v ->
-    maximum . map (\t -> ((s `atParam` t) <.> v) / magnitudeSq v) $ [0,1]
+    maximum (map (\t -> ((s `atParam` t) <.> v)) [0,1]) / magnitudeSq v
 
   getEnvelope (s@(Cubic c1 c2 (OffsetClosed x2))) = mkEnvelope $ \v ->
     maximum .
@@ -349,10 +351,10 @@ instance (InnerSpace v, OrderedField (Scalar v)) => Enveloped (FixedSegment v) w
 
 -- | Create a 'FixedSegment' from a located 'Segment'.
 mkFixedSeg :: AdditiveGroup v => Located (Segment Closed v) -> FixedSegment v
-mkFixedSeg (viewLoc -> (p, Linear (OffsetClosed v)))
-  = FLinear p (p .+^ v)
-mkFixedSeg (viewLoc -> (p, Cubic c1 c2 (OffsetClosed x2)))
-  = FCubic  p (p .+^ c1) (p .+^ c2) (p .+^ x2)
+mkFixedSeg ls =
+  case viewLoc ls of
+    (p, Linear (OffsetClosed v))       -> FLinear p (p .+^ v)
+    (p, Cubic c1 c2 (OffsetClosed x2)) -> FCubic  p (p .+^ c1) (p .+^ c2) (p .+^ x2)
 
 -- | Convert a 'FixedSegment' back into a located 'Segment'.
 fromFixedSeg :: AdditiveGroup v => FixedSegment v -> Located (Segment Closed v)
@@ -373,6 +375,35 @@ instance VectorSpace v => Parametric (FixedSegment v) where
 
           p3  = alerp p21 p22 t
 
+instance Num (Scalar v) => DomainBounds (FixedSegment v)
+
+instance (VectorSpace v, Num (Scalar v)) => EndValues (FixedSegment v) where
+  atStart (FLinear p0 _)     = p0
+  atStart (FCubic  p0 _ _ _) = p0
+  atEnd   (FLinear _ p1)     = p1
+  atEnd   (FCubic _ _ _ p1 ) = p1
+
+instance (VectorSpace v, Fractional (Scalar v)) => Sectionable (FixedSegment v) where
+  splitAtParam (FLinear p0 p1) t = (left, right)
+    where left  = FLinear p0 p
+          right = FLinear p  p1
+          p = alerp p0 p1 t
+  splitAtParam (FCubic p0 c1 c2 p1) t = (left, right)
+    where left  = FCubic p0 a b cut
+          right = FCubic cut c d p1
+          -- first round
+          a   = alerp p0 c1 t
+          p   = alerp c1 c2 t
+          d   = alerp c2 p1 t
+          -- second round
+          b   = alerp a  p  t
+          c   = alerp p  d  t
+          -- final round
+          cut = alerp b  c  t
+
+  reverseDomain (FLinear p0 p1) = FLinear p1 p0
+  reverseDomain (FCubic p0 c1 c2 p1) = FCubic p1 c2 c1 p0
+
 ------------------------------------------------------------
 --  Segment measures  --------------------------------------
 ------------------------------------------------------------
@@ -382,26 +413,40 @@ instance VectorSpace v => Parametric (FixedSegment v) where
 -- automatically track various monoidal \"measures\" on segments.
 
 -- | A type to track the count of segments in a 'Trail'.
-newtype SegCount = SegCount { getSegCount :: Sum Int }
+newtype SegCount = SegCount (Sum Int)
   deriving (Semigroup, Monoid)
 
+instance Wrapped SegCount where
+    type Unwrapped SegCount = Sum Int
+    _Wrapped' = iso (\(SegCount x) -> x) SegCount
+
+instance Rewrapped SegCount SegCount
+
 -- | A type to represent the total arc length of a chain of
 --   segments. The first component is a \"standard\" arc length,
 --   computed to within a tolerance of @10e-6@.  The second component is
 --   a generic arc length function taking the tolerance as an
 --   argument.
-newtype ArcLength v = ArcLength
-  { getArcLength :: (Sum (Interval (Scalar v)), Scalar v -> Sum (Interval (Scalar v))) }
+
+newtype ArcLength v
+  = ArcLength (Sum (Interval (Scalar v)), Scalar v -> Sum (Interval (Scalar v)))
+
+instance Wrapped (ArcLength v) where
+    type Unwrapped (ArcLength v) =
+        (Sum (Interval (Scalar v)), Scalar v -> Sum (Interval (Scalar v)))
+    _Wrapped' = iso (\(ArcLength x) -> x) ArcLength
+
+instance Rewrapped (ArcLength v) (ArcLength v')
 
 -- | Project out the cached arc length, stored together with error
 --   bounds.
 getArcLengthCached :: ArcLength v -> Interval (Scalar v)
-getArcLengthCached = getSum . fst . getArcLength
+getArcLengthCached = getSum . fst . op ArcLength
 
 -- | Project out the generic arc length function taking the tolerance as
 --   an argument.
 getArcLengthFun :: ArcLength v -> Scalar v -> Interval (Scalar v)
-getArcLengthFun = fmap getSum . snd . getArcLength
+getArcLengthFun = fmap getSum . snd . op ArcLength
 
 -- | Given a specified tolerance, project out the cached arc length if
 --   it is accurate enough; otherwise call the generic arc length
@@ -418,7 +463,13 @@ deriving instance (Num (Scalar v), Ord (Scalar v)) => Monoid    (ArcLength v)
 
 -- | A type to represent the total cumulative offset of a chain of
 --   segments.
-newtype TotalOffset v = TotalOffset { getTotalOffset :: v }
+newtype TotalOffset v = TotalOffset v
+
+instance Wrapped (TotalOffset v) where
+    type Unwrapped (TotalOffset v) = v
+    _Wrapped' = iso (\(TotalOffset x) -> x) TotalOffset
+
+instance Rewrapped (TotalOffset v) (TotalOffset v')
 
 instance AdditiveGroup v => Semigroup (TotalOffset v) where
   TotalOffset v1 <> TotalOffset v2 = TotalOffset (v1 ^+^ v2)
@@ -432,15 +483,20 @@ instance AdditiveGroup v => Monoid (TotalOffset v) where
 --   combining the envelopes of two consecutive chains needs to take
 --   the offset of the the offset of the first into account.
 data OffsetEnvelope v = OffsetEnvelope
-  { oeOffset   :: TotalOffset v
-  , oeEnvelope :: Envelope v
+  { _oeOffset   :: !(TotalOffset v)
+  , _oeEnvelope :: Envelope v
   }
 
+makeLenses ''OffsetEnvelope
+
 instance (InnerSpace v, OrderedField (Scalar v)) => Semigroup (OffsetEnvelope v) where
   (OffsetEnvelope o1 e1) <> (OffsetEnvelope o2 e2)
-    = OffsetEnvelope
-        (o1 <> o2)
-        (e1 <> moveOriginBy (negateV . getTotalOffset $ o1) e2)
+    = let !negOff = negateV . op TotalOffset $ o1
+          e2Off = moveOriginBy negOff e2
+          !_unused = maybe () (\f -> f `seq` ()) $ appEnvelope e2Off
+      in OffsetEnvelope
+          (o1 <> o2)
+          (e1 <> e2Off)
 
 -- | @SegMeasure@ collects up all the measurements over a chain of
 --   segments.
