@@ -7,10 +7,15 @@
 {-# LANGUAGE MultiParamTypeClasses      #-}
 {-# LANGUAGE ScopedTypeVariables        #-}
 {-# LANGUAGE StandaloneDeriving         #-}
+{-# LANGUAGE TemplateHaskell            #-}
 {-# LANGUAGE TypeFamilies               #-}
 {-# LANGUAGE TypeOperators              #-}
 {-# LANGUAGE UndecidableInstances       #-}
 {-# LANGUAGE ViewPatterns               #-}
+
+{-# OPTIONS_GHC -fno-warn-orphans #-}
+-- We have an orphan Transformable FingerTree instance here.
+
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Diagrams.Trail
@@ -72,7 +77,7 @@ module Diagrams.Trail
        , trailOffsets, trailOffset
        , lineOffsets, lineOffset, loopOffsets
        , trailVertices, lineVertices, loopVertices
-       , fixTrail
+       , trailLocSegments, fixTrail
 
          -- * Modifying trails
 
@@ -91,9 +96,14 @@ module Diagrams.Trail
 
        , SegTree(..), trailMeasure, numSegs, offset
 
+         -- ** Extracting segments
+
+       , GetSegment(..), getSegment
+
        ) where
 
 import           Control.Arrow       ((***))
+import           Control.Lens        (AnIso', iso, view, op, Wrapped(..), Rewrapped)
 import           Data.AffineSpace
 import           Data.FingerTree     (FingerTree, ViewL (..), ViewR (..), (<|),
                                       (|>))
@@ -102,7 +112,7 @@ import qualified Data.Foldable       as F
 import           Data.Monoid.MList
 import           Data.Semigroup
 import           Data.VectorSpace    hiding (Sum (..))
-import qualified Numeric.Interval    as I
+import qualified Numeric.Interval.Kaucher    as I
 
 import           Diagrams.Core       hiding ((|>))
 import           Diagrams.Located
@@ -137,10 +147,16 @@ instance ( HasLinearMap (V a), InnerSpace (V a), OrderedField (Scalar (V a))
 --   also easily slice and dice them according to the measures
 --   (/e.g./, split off the smallest number of segments from the
 --   beginning which have a combined arc length of at least 5).
-newtype SegTree v = SegTree
-                  { getSegTree :: FingerTree (SegMeasure v) (Segment Closed v) }
+
+newtype SegTree v = SegTree (FingerTree (SegMeasure v) (Segment Closed v))
   deriving (Eq, Ord, Show)
 
+instance Wrapped (SegTree v) where
+    type Unwrapped (SegTree v) = FingerTree (SegMeasure v) (Segment Closed v)
+    _Wrapped' = iso (\(SegTree x) -> x) SegTree
+
+instance Rewrapped (SegTree v) (SegTree v')
+
 type instance V (SegTree v) = v
 
 deriving instance (OrderedField (Scalar v), InnerSpace v)
@@ -181,7 +197,7 @@ instance (InnerSpace v, RealFrac (Scalar v), Floating (Scalar v))
     | otherwise = case FT.viewl after of
                     EmptyL    -> emptySplit
                     seg :< after' ->
-                      case seg `splitAtParam` (snd . properFraction $ p * tSegs) of
+                      case seg `splitAtParam` (snd . propFrac $ p * tSegs) of
                         (seg1, seg2) -> ( SegTree $ before |> seg1
                                         , SegTree $ seg2   <| after'
                                         )
@@ -190,6 +206,9 @@ instance (InnerSpace v, RealFrac (Scalar v), Floating (Scalar v))
       tSegs           = numSegs t
       emptySplit      = (SegTree t, SegTree t)
 
+  reverseDomain (SegTree t) = SegTree $ FT.reverse t'
+    where t' = FT.fmap' reverseSegment t
+
   -- XXX seems like it should be possible to collapse some of the
   -- above cases into one?
 
@@ -220,7 +239,7 @@ instance (InnerSpace v, OrderedField (Scalar v), RealFrac (Scalar v))
                          in  (p - 1)/tSegs + 1
     | otherwise    = case FT.viewl after of
                        EmptyL    -> 0
-                       seg :< after' ->
+                       seg :< _  ->
                          let p = arcLengthToParam (eps/2) seg
                                    (l - arcLength (eps/2) (SegTree before))
                          in  (numSegs before + p) / tSegs
@@ -248,14 +267,14 @@ numSegs :: ( Floating (Scalar v), Num c, Ord (Scalar v), InnerSpace v,
              FT.Measured (SegMeasure v) a
            )
         => a -> c
-numSegs = fromIntegral . trailMeasure 0 (getSum . getSegCount)
+numSegs = fromIntegral . trailMeasure 0 (getSum . op SegCount)
 
 -- | Compute the total offset of anything measured by 'SegMeasure'.
 offset :: ( Floating (Scalar v), Ord (Scalar v), InnerSpace v,
             FT.Measured (SegMeasure v) t
           )
        => t -> v
-offset = trailMeasure zeroV (getTotalOffset . oeOffset)
+offset = trailMeasure zeroV (op TotalOffset . view oeOffset)
 
 ------------------------------------------------------------
 --  Trails  ------------------------------------------------
@@ -330,8 +349,8 @@ data Trail' l v where
 -- | A generic eliminator for 'Trail'', taking functions specifying
 --   what to do in the case of a line or a loop.
 withTrail' :: (Trail' Line v -> r) -> (Trail' Loop v -> r) -> Trail' l v -> r
-withTrail' line loop t@(Line{}) = line t
-withTrail' line loop t@(Loop{}) = loop t
+withTrail' line _    t@(Line{}) = line t
+withTrail' _    loop t@(Loop{}) = loop t
 
 deriving instance Show v => Show (Trail' l v)
 deriving instance Eq   v => Eq   (Trail' l v)
@@ -361,7 +380,7 @@ instance (InnerSpace v, OrderedField (Scalar v)) => Enveloped (Trail' l v) where
   getEnvelope = withTrail' ftEnv (ftEnv . cutLoop)
     where
       ftEnv :: Trail' Line v -> Envelope v
-      ftEnv (Line t) = trailMeasure mempty oeEnvelope $ t
+      ftEnv (Line t) = trailMeasure mempty (view oeEnvelope) $ t
 
 instance (HasLinearMap v, InnerSpace v, OrderedField (Scalar v))
     => Renderable (Trail' o v) NullBackend where
@@ -371,12 +390,21 @@ instance (InnerSpace v, OrderedField (Scalar v), RealFrac (Scalar v))
     => Parametric (Trail' l v) where
   atParam t p = withTrail'
                   (\(Line segT) -> segT `atParam` p)
-                  (\l -> cutLoop l `atParam` p')
+                  (\l -> cutLoop l `atParam` mod1 p)
                   t
-    where
-      pf = snd . properFraction $ p
-      p' | p >= 0    = pf
-         | otherwise = 1 + pf
+
+-- | Compute the remainder mod 1.  Convenient for constructing loop
+--   parameterizations that wrap around.
+mod1 :: RealFrac a => a -> a
+mod1 p = p'
+ where
+   pf = snd . propFrac $ p
+   p' | p >= 0    = pf
+      | otherwise = 1 + pf
+
+-- Get rid of defaulting warnings
+propFrac :: RealFrac a => a -> (Int, a)
+propFrac = properFraction
 
 instance Num (Scalar v) => DomainBounds (Trail' l v)
 
@@ -389,6 +417,8 @@ instance (InnerSpace v, RealFrac (Scalar v), Floating (Scalar v))
     where
       (t1, t2) = splitAtParam t p
 
+  reverseDomain = reverseLine
+
 instance (InnerSpace v, OrderedField (Scalar v), RealFrac (Scalar v))
     => HasArcLength (Trail' l v) where
   arcLengthBounded eps =
@@ -402,6 +432,137 @@ instance (InnerSpace v, OrderedField (Scalar v), RealFrac (Scalar v))
       (\lp -> arcLengthToParam eps (cutLoop lp) l)
       tr
 
+--------------------------------------------------
+-- Extracting segments
+
+-- | A newtype wrapper around trails which exists solely for its
+--   'Parametric', 'DomainBounds' and 'EndValues' instances.  The idea
+--   is that if @tr@ is a trail, you can write, /e.g./
+--
+--   @
+--   getSegment tr `atParam` 0.6
+--   @
+--
+--   or
+--
+--   @
+--   atStart (getSegment tr)
+--   @
+--
+--   to get the segment at parameter 0.6 or the first segment in the
+--   trail, respectively.
+--
+--   The codomain for 'GetSegment', /i.e./ the result you get from
+--   calling 'atParam', 'atStart', or 'atEnd', is @Maybe (v, Segment
+--   Closed v, AnIso' (Scalar v) (Scalar v))@.  @Nothing@ results if
+--   the trail is empty; otherwise, you get:
+--
+--   * the offset from the start of the trail to the beginning of the
+--     segment,
+--
+--   * the segment itself, and
+--
+--   * a reparameterization isomorphism: in the forward direction, it
+--     translates from parameters on the whole trail to a parameters
+--     on the segment.  Note that for technical reasons you have to
+--     call 'cloneIso' on the @AnIso'@ value to get a real isomorphism
+--     you can use.
+newtype GetSegment t = GetSegment t
+
+-- | Create a 'GetSegment' wrapper around a trail, after which you can
+--   call 'atParam', 'atStart', or 'atEnd' to extract a segment.
+getSegment :: t -> GetSegment t
+getSegment = GetSegment
+
+type instance V (GetSegment t) = V t
+type instance Codomain (GetSegment t)
+  = Maybe
+    ( V t                                   -- offset from trail start to segment start
+    , Segment Closed (V t)                  -- the segment
+    , AnIso' (Scalar (V t)) (Scalar (V t))  -- reparameterization, trail <-> segment
+    )
+
+-- | Parameters less than 0 yield the first segment; parameters
+--   greater than 1 yield the last.  A parameter exactly at the
+--   junction of two segments yields the second segment (/i.e./ the
+--   one with higher parameter values).
+instance (InnerSpace v, OrderedField (Scalar v))
+    => Parametric (GetSegment (Trail' Line v)) where
+  atParam (GetSegment (Line (SegTree ft))) p
+    | p <= 0
+    = case FT.viewl ft of
+        EmptyL    -> Nothing
+        seg :< _  -> Just (zeroV, seg, reparam 0)
+
+    | p >= 1
+    = case FT.viewr ft of
+        EmptyR     -> Nothing
+        ft' :> seg -> Just (offset ft', seg, reparam (n-1))
+
+    | otherwise
+    = let (before, after) = FT.split ((p*n <) . numSegs) $ ft
+      in  case FT.viewl after of
+            EmptyL   -> Nothing
+            seg :< _ -> Just (offset before, seg, reparam (numSegs before))
+    where
+      n = numSegs ft
+      reparam k = iso (subtract k . (*n))
+                      ((/n) . (+ k))
+
+-- | The parameterization for loops wraps around, /i.e./ parameters
+--   are first reduced \"mod 1\".
+instance (InnerSpace v, OrderedField (Scalar v), RealFrac (Scalar v))
+    => Parametric (GetSegment (Trail' Loop v)) where
+  atParam (GetSegment l) p = atParam (GetSegment (cutLoop l)) (mod1 p)
+
+instance (InnerSpace v, OrderedField (Scalar v), RealFrac (Scalar v))
+    => Parametric (GetSegment (Trail v)) where
+  atParam (GetSegment t) p
+    = withTrail
+      ((`atParam` p) . GetSegment)
+      ((`atParam` p) . GetSegment)
+      t
+
+instance DomainBounds t => DomainBounds (GetSegment t) where
+  domainLower (GetSegment t) = domainLower t
+  domainUpper (GetSegment t) = domainUpper t
+
+instance (InnerSpace v, OrderedField (Scalar v))
+    => EndValues (GetSegment (Trail' Line v)) where
+  atStart (GetSegment (Line (SegTree ft)))
+    = case FT.viewl ft of
+        EmptyL   -> Nothing
+        seg :< _ ->
+          let n = numSegs ft
+          in  Just (zeroV, seg, iso (*n) (/n))
+
+  atEnd (GetSegment (Line (SegTree ft)))
+    = case FT.viewr ft of
+        EmptyR     -> Nothing
+        ft' :> seg ->
+          let n = numSegs ft
+          in  Just (offset ft', seg, iso (subtract (n-1) . (*n))
+                                         ((/n) . (+ (n-1)))
+                   )
+
+instance (InnerSpace v, OrderedField (Scalar v), RealFrac (Scalar v))
+    => EndValues (GetSegment (Trail' Loop v)) where
+  atStart (GetSegment l) = atStart (GetSegment (cutLoop l))
+  atEnd   (GetSegment l) = atEnd   (GetSegment (cutLoop l))
+
+instance (InnerSpace v, OrderedField (Scalar v), RealFrac (Scalar v))
+    => EndValues (GetSegment (Trail v)) where
+  atStart (GetSegment t)
+    = withTrail
+      (\l -> atStart (GetSegment l))
+      (\l -> atStart (GetSegment l))
+      t
+  atEnd (GetSegment t)
+    = withTrail
+      (\l -> atEnd (GetSegment l))
+      (\l -> atEnd (GetSegment l))
+      t
+
 --------------------------------------------------
 -- The Trail type
 
@@ -484,6 +645,8 @@ instance (InnerSpace v, RealFrac (Scalar v), Floating (Scalar v))
     => Sectionable (Trail v) where
   splitAtParam t p = withLine ((wrapLine *** wrapLine) . (`splitAtParam` p)) t
 
+  reverseDomain = reverseTrail
+
 instance (InnerSpace v, OrderedField (Scalar v), RealFrac (Scalar v))
     => HasArcLength (Trail v) where
   arcLengthBounded = withLine . arcLengthBounded
@@ -575,10 +738,10 @@ trailFromSegments = wrapTrail . lineFromSegments
 --   vectors, where each vector represents the offset from one vertex
 --   to the next.  See also 'fromOffsets'.
 --
---   <<diagrams/lineFromOffsetsEx.svg#diagram=lineFromOffsetsEx&width=300>>
+--   <<diagrams/src_Diagrams_Trail_lineFromOffsetsEx.svg#diagram=lineFromOffsetsEx&width=300>>
 --
 --   > import Diagrams.Coordinates
---   > lineFromOffsetsEx = strokeLine $ lineFromOffsets [ 2 & 1, 2 & (-1), 2 & 0.5 ]
+--   > lineFromOffsetsEx = strokeLine $ lineFromOffsets [ 2 ^& 1, 2 ^& (-1), 2 ^& 0.5 ]
 lineFromOffsets :: (InnerSpace v, OrderedField (Scalar v))
                   => [v] -> Trail' Line v
 lineFromOffsets = lineFromSegments . map straight
@@ -603,11 +766,11 @@ trailFromOffsets = wrapTrail . lineFromOffsets
 --   construct, say, a @'Located' ('Trail'' 'Line' v)@ or a @'Located'
 --   ('Trail' v)@.
 --
---   <<diagrams/lineFromVerticesEx.svg#diagram=lineFromVerticesEx&width=300>>
+--   <<diagrams/src_Diagrams_Trail_lineFromVerticesEx.svg#diagram=lineFromVerticesEx&width=300>>
 --
 --   > import Diagrams.Coordinates
 --   > lineFromVerticesEx = pad 1.1 . centerXY . strokeLine
---   >   $ lineFromVertices [origin, 0 & 1, 1 & 2, 5 & 1]
+--   >   $ lineFromVertices [origin, 0 ^& 1, 1 ^& 2, 5 ^& 1]
 lineFromVertices :: (InnerSpace v, OrderedField (Scalar v))
                    => [Point v] -> Trail' Line v
 lineFromVertices []  = emptyLine
@@ -632,14 +795,13 @@ trailFromVertices = wrapTrail . lineFromVertices
 --   know happens to end where it starts, and then call 'glueLine' to
 --   turn it into a loop.
 --
---   <<diagrams/glueLineEx.svg#diagram=glueLineEx&width=500>>
+--   <<diagrams/src_Diagrams_Trail_glueLineEx.svg#diagram=glueLineEx&width=500>>
 --
---   > import Diagrams.Coordinates
---   > glueLineEx = pad 1.1 . hcat' with {sep = 1}
+--   > glueLineEx = pad 1.1 . hcat' (with & sep .~ 1)
 --   >   $ [almostClosed # strokeLine, almostClosed # glueLine # strokeLoop]
 --   >
 --   > almostClosed :: Trail' Line R2
---   > almostClosed = fromOffsets [2 & (-1), (-3) & (-0.5), (-2) & 1, 1 & 0.5]
+--   > almostClosed = fromOffsets $ map r2 [(2, -1), (-3, -0.5), (-2, 1), (1, 0.5)]
 --
 --   @glueLine@ is left inverse to 'cutLoop', that is,
 --
@@ -677,9 +839,9 @@ glueTrail = onTrail glueLine id
 -- closeLine . lineFromVertices $ ps
 -- @
 --
---   <<diagrams/closeLineEx.svg#diagram=closeLineEx&width=500>>
+--   <<diagrams/src_Diagrams_Trail_closeLineEx.svg#diagram=closeLineEx&width=500>>
 --
---   > closeLineEx = pad 1.1 . centerXY . hcat' with {sep = 1}
+--   > closeLineEx = pad 1.1 . centerXY . hcat' (with & sep .~ 1)
 --   >   $ [almostClosed # strokeLine, almostClosed # closeLine # strokeLoop]
 closeLine :: Trail' Line v -> Trail' Loop v
 closeLine (Line t) = Loop t (Linear OffsetOpen)
@@ -707,7 +869,7 @@ cutLoop (Loop (SegTree t) c) =
     (_   , Cubic c1 c2 OffsetOpen) -> Line (SegTree (t |> Cubic c1 c2 off))
   where
     offV :: v
-    offV = negateV . trailMeasure zeroV (getTotalOffset . oeOffset) $ t
+    offV = negateV . trailMeasure zeroV (op TotalOffset .view oeOffset) $ t
     off = OffsetClosed offV
 
 -- | @cutTrail@ is a variant of 'cutLoop' for 'Trail'; it is the is
@@ -770,11 +932,11 @@ trailOffsets = withLine lineOffsets
 --
 --   but is more efficient.
 --
---   <<diagrams/trailOffsetEx.svg#diagram=trailOffsetEx&width=300>>
+--   <<diagrams/src_Diagrams_Trail_trailOffsetEx.svg#diagram=trailOffsetEx&width=300>>
 --
 --   > trailOffsetEx = (strokeLine almostClosed <> showOffset) # centerXY # pad 1.1
 --   >   where showOffset = fromOffsets [trailOffset (wrapLine almostClosed)]
---   >                    # stroke # lc red # lw 0.05
+--   >                    # stroke # lc red
 trailOffset :: (InnerSpace v, OrderedField (Scalar v)) => Trail v -> v
 trailOffset = withLine lineOffset
 
@@ -790,7 +952,7 @@ loopOffsets = lineOffsets . cutLoop
 --   there is no corresponding @loopOffset@ function because by
 --   definition it would be constantly zero.)
 lineOffset :: (InnerSpace v, OrderedField (Scalar v)) => Trail' Line v -> v
-lineOffset (Line t) = trailMeasure zeroV (getTotalOffset . oeOffset) t
+lineOffset (Line t) = trailMeasure zeroV (op TotalOffset . view oeOffset) t
 
 -- | Extract the vertices of a concretely located trail.  Note that
 --   for loops, the starting vertex will /not/ be repeated at the end.
@@ -802,7 +964,7 @@ lineOffset (Line t) = trailMeasure zeroV (getTotalOffset . oeOffset) t
 --   Note that it does not make sense to ask for the vertices of a
 --   'Trail' by itself; if you want the vertices of a trail
 --   with the first vertex at, say, the origin, you can use
---   @trailVertices . (`at` origin)@.
+--   @trailVertices . (\`at\` origin)@.
 trailVertices :: (InnerSpace v, OrderedField (Scalar v))
               => Located (Trail v) -> [Point v]
 trailVertices (viewLoc -> (p,t))
@@ -829,8 +991,12 @@ segmentVertices p = scanl (.+^) p . map segOffset
 -- | Convert a concretely located trail into a list of fixed segments.
 fixTrail :: (InnerSpace v, OrderedField (Scalar v))
          => Located (Trail v) -> [FixedSegment v]
-fixTrail t = zipWith ((mkFixedSeg .) . at)
-               (trailSegments (unLoc t)) (trailVertices t)
+fixTrail t = map mkFixedSeg (trailLocSegments t)
+
+-- | Convert a concretely located trail into a list of located segments.
+trailLocSegments :: (InnerSpace v, OrderedField (Scalar v))
+                  => Located (Trail v) -> [Located (Segment Closed v)]
+trailLocSegments t = zipWith at (trailSegments (unLoc t)) (trailVertices t)
 
 ------------------------------------------------------------
 --  Modifying trails  --------------------------------------
