@@ -3,6 +3,8 @@
 {-# LANGUAGE FlexibleInstances          #-}
 {-# LANGUAGE GeneralizedNewtypeDeriving #-}
 {-# LANGUAGE MultiParamTypeClasses      #-}
+{-# LANGUAGE Rank2Types                 #-}
+{-# LANGUAGE TemplateHaskell            #-}
 {-# LANGUAGE TypeFamilies               #-}
 {-# LANGUAGE ViewPatterns               #-}
 {-# OPTIONS_GHC -fno-warn-orphans #-}
@@ -23,14 +25,15 @@
 module Diagrams.TwoD.Path
        ( -- * Constructing path-based diagrams
 
-         stroke, stroke', strokeT, strokeT', strokeLine, strokeLoop
-       , strokeLocT, strokeLocLine, strokeLocLoop
+         stroke, stroke', strokeTrail, strokeT, strokeTrail', strokeT'
+       , strokeLine, strokeLoop
+       , strokeLocTrail, strokeLocT, strokeLocLine, strokeLocLoop
 
          -- ** Stroke options
 
        , FillRule(..)
        , FillRuleA(..), getFillRule, fillRule
-       , StrokeOpts(..)
+       , StrokeOpts(..), vertexNames, queryFillRule
 
          -- ** Inside/outside testing
 
@@ -38,10 +41,13 @@ module Diagrams.TwoD.Path
 
          -- * Clipping
 
-       , Clip(..), clipBy
+       , Clip(..), clipBy, clipTo, clipped
        ) where
 
 import           Control.Applicative   (liftA2)
+import           Control.Lens          (Lens, Lens', generateSignatures,
+                                        lensRules, makeLensesWith, makeWrapped,
+                                        op, (.~), (^.), _Wrapped')
 import qualified Data.Foldable         as F
 import           Data.Semigroup
 import           Data.Typeable
@@ -50,8 +56,10 @@ import           Data.AffineSpace
 import           Data.Default.Class
 import           Data.VectorSpace
 
+import           Diagrams.Combinators  (withEnvelope, withTrace)
 import           Diagrams.Coordinates
 import           Diagrams.Core
+import           Diagrams.Core.Trace
 import           Diagrams.Located      (Located, mapLoc, unLoc)
 import           Diagrams.Parametric
 import           Diagrams.Path
@@ -59,7 +67,7 @@ import           Diagrams.Segment
 import           Diagrams.Solve
 import           Diagrams.Trail
 import           Diagrams.TrailLike
-import           Diagrams.TwoD.Segment
+import           Diagrams.TwoD.Segment ()
 import           Diagrams.TwoD.Types
 import           Diagrams.Util         (tau)
 
@@ -79,12 +87,66 @@ instance Traced (Trail R2) where
     . lineSegments
 
 instance Traced (Path R2) where
-  getTrace = F.foldMap getTrace . pathTrails
+  getTrace = F.foldMap getTrace . op Path
 
 ------------------------------------------------------------
 --  Constructing path-based diagrams  ----------------------
 ------------------------------------------------------------
 
+-- | Enumeration of algorithms or \"rules\" for determining which
+--   points lie in the interior of a (possibly self-intersecting)
+--   closed path.
+data FillRule = Winding  -- ^ Interior points are those with a nonzero
+                         --   /winding/ /number/.  See
+                         --   <http://en.wikipedia.org/wiki/Nonzero-rule>.
+              | EvenOdd  -- ^ Interior points are those where a ray
+                         --   extended infinitely in a particular
+                         --   direction crosses the path an odd number
+                         --   of times. See
+                         --   <http://en.wikipedia.org/wiki/Even-odd_rule>.
+    deriving (Eq, Show)
+
+instance Default FillRule where
+  def = Winding
+
+-- | A record of options that control how a path is stroked.
+--   @StrokeOpts@ is an instance of 'Default', so a @StrokeOpts@
+--   records can be created using @'with' { ... }@ notation.
+data StrokeOpts a
+  = StrokeOpts
+    { _vertexNames   :: [[a]]
+
+    , _queryFillRule :: FillRule
+
+    }
+
+makeLensesWith (generateSignatures .~ False $ lensRules) ''StrokeOpts
+
+-- | Atomic names that should be assigned to the vertices of the path so that
+--   they can be referenced later.  If there are not enough names, the extra
+--   vertices are not assigned names; if there are too many, the extra names
+--   are ignored.  Note that this is a /list of lists/ of names, since paths
+--   can consist of multiple trails.  The first list of names are assigned to
+--   the vertices of the first trail, the second list to the second trail, and
+--   so on.
+--
+--   The default value is the empty list.
+
+vertexNames :: forall a a'. Lens (StrokeOpts a) (StrokeOpts a') [[a]] [[a']]
+
+-- | The fill rule used for determining which points are inside the path.
+--   The default is 'Winding'.  NOTE: for now, this only affects the resulting
+--   diagram's 'Query', /not/ how it will be drawn!  To set the fill rule
+--   determining how it is to be drawn, use the 'fillRule' function.
+queryFillRule :: forall a. Lens' (StrokeOpts a) FillRule
+
+
+instance Default (StrokeOpts a) where
+  def = StrokeOpts
+        { _vertexNames    = []
+        , _queryFillRule = def
+        }
+
 -- | Convert a path into a diagram.  The resulting diagram has the
 --   names 0, 1, ... assigned to each of the path's vertices.
 --
@@ -107,79 +169,50 @@ instance Renderable (Path R2) b => TrailLike (QDiagram b R2 Any) where
 --
 --     * Names can be assigned to the path's vertices
 --
---   'StrokeOpts' is an instance of 'Default', so @stroke' 'with' {
---   ... }@ syntax may be used.
+--   'StrokeOpts' is an instance of 'Default', so @stroke' ('with' &
+--   ... )@ syntax may be used.
 stroke' :: (Renderable (Path R2) b, IsName a) => StrokeOpts a -> Path R2 -> Diagram b R2
 stroke' opts path
-  | null (pathTrails p1) =           mkP p2
-  | null (pathTrails p2) = mkP p1
-  | otherwise            = mkP p1 <> mkP p2
+  | null (pLines ^. _Wrapped') =           mkP pLoops
+  | null (pLoops ^. _Wrapped') = mkP pLines
+  | otherwise                   = mkP pLines <> mkP pLoops
   where
-    (p1,p2) = partitionPath (isLine . unLoc) path
+    (pLines,pLoops) = partitionPath (isLine . unLoc) path
     mkP p
       = mkQD (Prim p)
          (getEnvelope p)
          (getTrace p)
          (fromNames . concat $
-           zipWith zip (vertexNames opts) ((map . map) subPoint (pathVertices p))
+           zipWith zip (opts^.vertexNames) ((map . map) subPoint (pathVertices p))
          )
-         (Query $ Any . flip (runFillRule (queryFillRule opts)) p)
+         (Query $ Any . flip (runFillRule (opts^.queryFillRule)) p)
 
--- | A record of options that control how a path is stroked.
---   @StrokeOpts@ is an instance of 'Default', so a @StrokeOpts@
---   records can be created using @'with' { ... }@ notation.
-data StrokeOpts a
-  = StrokeOpts
-    { vertexNames   :: [[a]]  -- ^ Atomic names that should be assigned
-                            --   to the vertices of the path so that
-                            --   they can be referenced later.  If
-                            --   there are not enough names, the extra
-                            --   vertices are not assigned names; if
-                            --   there are too many, the extra names
-                            --   are ignored.  Note that this is a
-                            --   /list of lists/ of names, since paths
-                            --   can consist of multiple trails.  The
-                            --   first list of names are assigned to
-                            --   the vertices of the first trail, the
-                            --   second list to the second trail, and
-                            --   so on.
-                            --
-                            --   The default value is the empty list.
-
-    , queryFillRule :: FillRule
-                            -- ^ The fill rule used for determining
-                            --   which points are inside the path.
-                            --   The default is 'Winding'.  NOTE: for
-                            --   now, this only affects the resulting
-                            --   diagram's 'Query', /not/ how it will
-                            --   be drawn!  To set the fill rule
-                            --   determining how it is to be drawn,
-                            --   use the 'fillRule' function.
-    }
-
-instance Default (StrokeOpts a) where
-  def = StrokeOpts
-        { vertexNames    = []
-        , queryFillRule = Winding
-        }
 
 -- | A composition of 'stroke' and 'pathFromTrail' for conveniently
 --   converting a trail directly into a diagram.
 --
 --   Note that a bug in GHC 7.0.1 causes a context stack overflow when
---   inferring the type of 'stroke' and hence of @strokeT@ as well.
+--   inferring the type of 'stroke' and hence of @strokeTrail@ as well.
 --   The solution is to give a type signature to expressions involving
---   @strokeT@, or (recommended) upgrade GHC (the bug is fixed in 7.0.2
+--   @strokeTrail@, or (recommended) upgrade GHC (the bug is fixed in 7.0.2
 --   onwards).
-strokeT :: (Renderable (Path R2) b)
-        => Trail R2 -> Diagram b R2
-strokeT = stroke . pathFromTrail
+strokeTrail :: (Renderable (Path R2) b) => Trail R2 -> Diagram b R2
+strokeTrail = stroke . pathFromTrail
+
+-- | Deprecated synonym for 'strokeTrail'.
+strokeT :: (Renderable (Path R2) b) => Trail R2 -> Diagram b R2
+strokeT = strokeTrail
 
 -- | A composition of 'stroke'' and 'pathFromTrail' for conveniently
 --   converting a trail directly into a diagram.
+strokeTrail' :: (Renderable (Path R2) b, IsName a)
+             => StrokeOpts a -> Trail R2 -> Diagram b R2
+strokeTrail' opts = stroke' opts . pathFromTrail
+
+-- | Deprecated synonym for 'strokeTrail''.
 strokeT' :: (Renderable (Path R2) b, IsName a)
          => StrokeOpts a -> Trail R2 -> Diagram b R2
-strokeT' opts = stroke' opts . pathFromTrail
+strokeT' = strokeTrail'
 
 -- | A composition of 'strokeT' and 'wrapLine' for conveniently
 --   converting a line directly into a diagram.
@@ -192,9 +225,13 @@ strokeLoop :: (Renderable (Path R2) b) => Trail' Loop R2 -> Diagram b R2
 strokeLoop = strokeT . wrapLoop
 
 -- | A convenience function for converting a @Located Trail@ directly
---   into a diagram; @strokeLocT = stroke . trailLike@.
+--   into a diagram; @strokeLocTrail = stroke . trailLike@.
+strokeLocTrail :: (Renderable (Path R2) b) => Located (Trail R2) -> Diagram b R2
+strokeLocTrail = stroke . trailLike
+
+-- | Deprecated synonym for 'strokeLocTrail'.
 strokeLocT :: (Renderable (Path R2) b) => Located (Trail R2) -> Diagram b R2
-strokeLocT = stroke . trailLike
+strokeLocT = strokeLocTrail
 
 -- | A convenience function for converting a @Located@ line directly
 --   into a diagram; @strokeLocLine = stroke . trailLike . mapLoc wrapLine@.
@@ -210,27 +247,19 @@ strokeLocLoop = stroke . trailLike . mapLoc wrapLoop
 --  Inside/outside testing
 ------------------------------------------------------------
 
--- | Enumeration of algorithms or \"rules\" for determining which
---   points lie in the interior of a (possibly self-intersecting)
---   closed path.
-data FillRule = Winding  -- ^ Interior points are those with a nonzero
-                         --   /winding/ /number/.  See
-                         --   <http://en.wikipedia.org/wiki/Nonzero-rule>.
-              | EvenOdd  -- ^ Interior points are those where a ray
-                         --   extended infinitely in a particular
-                         --   direction crosses the path an odd number
-                         --   of times. See
-                         --   <http://en.wikipedia.org/wiki/Even-odd_rule>.
-    deriving (Eq)
+
 
 runFillRule :: FillRule -> P2 -> Path R2 -> Bool
 runFillRule Winding = isInsideWinding
 runFillRule EvenOdd = isInsideEvenOdd
 
 newtype FillRuleA = FillRuleA (Last FillRule)
-  deriving (Typeable, Semigroup)
+  deriving (Typeable, Semigroup, Show)
 instance AttributeClass FillRuleA
 
+instance Default FillRuleA where
+  def = FillRuleA $ Last $ def
+
 -- | Extract the fill rule from a 'FillRuleA' attribute.
 getFillRule :: FillRuleA -> FillRule
 getFillRule (FillRuleA (Last r)) = r
@@ -263,7 +292,7 @@ isInsideEvenOdd p = odd . crossings p
 -- | Compute the sum of /signed/ crossings of a path as we travel in the
 --   positive x direction from a given point.
 crossings :: P2 -> Path R2 -> Int
-crossings p = F.sum . map (trailCrossings p) . pathTrails
+crossings p = F.sum . map (trailCrossings p) . op Path
 
 -- | Compute the sum of signed crossings of a trail starting from the
 --   given point in the positive x direction.
@@ -313,8 +342,11 @@ trailCrossings p@(unp2 -> (x,y)) tr
 --   concatenation, so applying multiple clipping paths is sensible.
 --   The clipping region is the intersection of all the applied
 --   clipping paths.
-newtype Clip = Clip { getClip :: [Path R2] }
+newtype Clip = Clip [Path R2]
   deriving (Typeable, Semigroup)
+
+makeWrapped ''Clip
+
 instance AttributeClass Clip
 
 type instance V Clip = R2
@@ -331,7 +363,30 @@ instance Transformable Clip where
 clipBy :: (HasStyle a, V a ~ R2) => Path R2 -> a -> a
 clipBy = applyTAttr . Clip . (:[])
 
--- XXX Should include a 'clipTo' function which clips a diagram AND
--- restricts its envelope.  It will have to take a *pointwise minimum*
--- of the diagram's current envelope and the path's envelope.  Not
--- sure of the best way to do this at the moment.
+-- | Clip a diagram to the given path setting its envelope to the
+--   pointwise minimum of the envelopes of the diagram and path. The
+--   trace consists of those parts of the original diagram's trace
+--   which fall within the clipping path, or parts of the path's trace
+--   within the original diagram.
+clipTo :: (Renderable (Path R2) b) => Path R2 ->  Diagram b R2 ->  Diagram b R2
+clipTo p d = setTrace intersectionTrace . toEnvelope $ clipBy p d
+  where
+    envP = appEnvelope . getEnvelope $ p
+    envD = appEnvelope . getEnvelope $ d
+    toEnvelope = case (envP, envD) of
+      (Just eP, Just eD) -> setEnvelope . mkEnvelope $ \v -> min (eP v) (eD v)
+      (_, _)             -> id
+    intersectionTrace = Trace intersections
+    intersections pt v =
+        -- on boundary of d, inside p
+        onSortedList (filter pInside) (appTrace (getTrace d) pt v) <>
+        -- or on boundary of p, inside d
+        onSortedList (filter dInside) (appTrace (getTrace p) pt v) where
+          newPt dist = pt .+^ v ^* dist
+          pInside dDist = runFillRule Winding (newPt dDist) p
+          dInside pDist = getAny . sample d $ newPt pDist
+
+-- | Clip a diagram to the clip path taking the envelope and trace of the clip
+--   path.
+clipped :: (Renderable (Path R2) b) => Path R2 ->  Diagram b R2 ->  Diagram b R2
+clipped p = (withTrace p) . (withEnvelope p) . (clipBy p)
